.386
_Text SEGMENT PUBLIC USE16
  assume CS:_Text, DS:_Text

CRLF MACRO
  push AX BX
  mov AX, 0E00h+13
  mov BX, 0
  int 10h          ;;Print CR
  mov AX, 0E00h+10
  int 10h          ;;Print LF
  pop BX AX
ENDM
WRITE MACRO String
  LOCAL AfterString, StringAddr
  push SI
  mov SI, OFFSET StringAddr
  call WriteString
  pop SI
  jmp AfterString
StringAddr db String, 0
AfterString:
ENDM
WRITELN MACRO String
  LOCAL AfterString, StringAddr
  push SI
  mov SI, OFFSET StringAddr
  call WriteString
  pop SI
  jmp AfterString
StringAddr db String, 13, 10, 0
AfterString:
ENDM
WriteLnWord MACRO WordMem:REST
  push AX BX
  mov AX, WordMem
  call WriteAX
  mov AX, 0E0Dh
  mov BX, 0
  int 10h             ;;Print CR
  mov AX, 0E0Ah
  int 10h             ;;Print LF
  pop BX AX
ENDM

  org 0
EntryPoint:
  jmp AfterData

;Put any data here!
HexTable            db '0123456789ABCDEF'
DiskHeads           dd ?
DiskTracksPerHead   dd ?
DiskSectorsPerTrack dd ?

DiskFatSectors      dw ?
DiskRootDirSector   dw ?

AfterData:
  push CS
  pop DS
  mov AX, 50h
  mov ES, AX      ;Sector Buffer

  WRITE "Boot code startup.  CS: "
  mov AX, CS
  call WriteAXHex
  CRLF
  CRLF

  WRITELN "Installing Interrupt 21h Handler"
  mov AX, CS
  shl EAX, 16
  mov AX, OFFSET Int21h
  mov BX, 0
  mov FS, BX
  cli
  mov FS:[21h*4], EAX
  sti

  WRITELN "Stats for Drive A:"
  mov DL, 0
  call GetDriveStats

  WRITE "Heads: "
  WriteLNWord WORD PTR [DiskHeads]

  WRITE "Tracks / Head: "
  WriteLNWord WORD PTR [DiskTracksPerHead]

  WRITE "Sectors / Track: "
  WriteLNWord WORD PTR [DiskSectorsPerTrack]

  mov BX, 0        ;ES is already set
  mov ECX, 1       ;Read Sector 1
  call ReadSector  ;Read the FAT part 1

  mov EAX, ES:[BX]  ;Read number of Allocation Entries
  shl EAX, 2        ;Each is 4 bytes long
  add EAX, 511
  shr EAX, 9        ;Root directory sector = (# of Entries*4+511) / 512
  mov [DiskFatSectors], AX
  inc EAX           ;Skip over boot sector
  mov [DiskRootDirSector], AX

  WRITE "Sectors occupied by the FAT: "
  WriteLNWord [DiskFatSectors]

  WRITE "First Sector of Root Directory: "
  WriteLNWord [DiskRootDirSector]

  mov BX, 0                    ;ES is already set
  xor ECX, ECX
  mov CX, [DiskRootDirSector]
  call ReadSector              ;Read the Root Directory 

  mov EAX, ES:[BX]
  WRITE "Number of Root directory entries: "
  call WriteAX
  CRLF
  mov ECX, EAX

  CRLF
  WRITELN "Root directory: -----------------------"
  push DS
  push ES
  pop DS
  mov SI, BX
  add SI, 12
PrintDirectoryLoop:
  add SI, 64

  call WriteString
  CRLF
  loop PrintDirectoryLoop
  pop DS
  xor BX, BX

  jmp $


Int21JumpTable dw OFFSET Service0, OFFSET Service1
LengthInt21JumpTable = ($-OFFSET Int21JumpTable) SHR 1

Int21h:
  push EAX EBX ECX EDX EBP ESP ESI EDI
  push DS ES FS GS

  cmp AX, LengthInt21JumpTable
  jae ServiceNA  ;Not available service!
  jmp CS:[Int21JumpTable+EAX*2]

CommonExit:
  pop GS FS ES DS
  pop EDI ESI ESP EBP EDX ECX EBX EAX
  iret

ServiceNA:
  WRITE "Invalid service #: "
  call WriteAX
  CRLF
  jmp CommonExit

Service0:
  WRITELN "Int21:  Service 0"
  jmp CommonExit

Service1:
  WRITELN "Int21:  Service 1"
  jmp CommonExit


TranslateLogicalSector:          ;In: ECX = Sector  Out: CH, CL, DH, DL
  push EAX EBX
  mov EAX, ECX
  xor EDX, EDX
  div [DiskSectorsPerTrack]
  mov CL, DL                     ;Set the sector
  and CL, 63                     ;Top two bits are bits 8&9 of the cylinder
  xor EDX, EDX
  div [DiskTracksPerHead]
  mov CH, DL                     ;Set the track bits 0-7
  mov AL, DH
  shl AL, 6
  or CL, AL                      ;Set bits 8&9 of track
  xor EDX, EDX
  div [DiskHeads]
  mov DH, DL                     ;Set the head
  inc CL                         ;Sectors start with 1, not 0
  pop EBX EAX
  ret

ReadSector:   ;In: ES:BX = Data buffer; ECX = Sector
  push AX CX DX SI
  call TranslateLogicalSector  ;In: ECX  Out: CX, DX
  mov SI, 4
ReadSectorLoop:
  mov AX, 201h  ;Service 2, Read 1 sector
  mov DL, 0     ;A: drive
  int 13h
  jnc DoneReadSector
  dec SI
  jnz ReadSectorLoop

  WRITELN "DISK READ ERROR"
  WRITE "CX: "
  WriteLnWord CX
  WRITE "DX: "
  WriteLnWord DX

DoneReadSector:
  pop SI DX CX AX
  ret

GetDriveStats:  ;In DL: Drive to get stats for
  mov AH, 8
  int 13h
  xor EAX, EAX
  mov [DiskHeads], EAX
  mov [DiskTracksPerHead], EAX
  mov [DiskSectorsPerTrack], EAX

  mov AL, DH               ;Set the Heads
  mov WORD PTR [DiskHeads], AX

  mov AL, CH               ;Tracks per Head
  mov BYTE PTR [DiskTracksPerHead], AL
  mov AL, CL
  shr AL, 6
  mov BYTE PTR [DiskTracksPerHead+1], AL

  mov AL, CL               ;Sectors Per Track
  and AL, 63
  mov [DiskSectorsPerTrack], EAX

  inc [DiskHeads]
  inc [DiskTracksPerHead]
  mov AX, 50h
  mov ES, AX      ;Sector Buffer
  ret

WriteString:
  push AX BX SI
WriteStringLoop:
  mov AL, DS:[SI]
  inc SI
  or AL, AL
  jz DoneWriteString

  mov BX, 0
  mov AH, 0Eh
  int 10h
  jmp WriteStringLoop
DoneWriteString:
  pop SI BX AX
  ret

WriteAX:
  push AX BX

Divider = 10000
REPT 5
  mov BX, Divider
  xor DX, DX
  div BX
  add AL, '0'
  mov AH, 0Eh
  mov BX, 0
  int 10h
  mov AX, DX
  Divider = Divider / 10
ENDM

  pop BX AX
  ret

WriteAXHex:
  push AX BX
  mov BL, AH
  shr BL, 4
  call WriteBLNibble
  mov BL, AH
  call WriteBLNibble
  mov BL, AL
  shr BL, 4
  call WriteBLNibble
  mov BL, AL
  call WriteBLNibble
  mov AX, 0E00h +'h'
  mov BX, 0
  int 10h
  pop BX AX
  ret

WriteBLNibble:
  push AX
  and BX, 0Fh
  mov AL, DS:[BX+OFFSET HexTable]
  mov AH, 0Eh
  mov BX, 0
  int 10h
  pop AX
;Done:
  ret

_Text ENDS
  END
