SectorBuffer   = 50h
BootCodeSector = 70h
.386
_Text SEGMENT PUBLIC USE16
  Assume CS:_text, DS:_text
CRLF MACRO
  push AX BX
  mov AX, 0E00h+13
  mov BX, 0
  int 10h    ;;Print CR
  mov AX, 0E00h+10
  int 10h    ;;Print LF
  pop BX AX
ENDM
WRITE MACRO String
  LOCAL AfterString, StringAddr
  jmp AfterString
StringAddr db String, 0
AfterString:
  push SI
  mov SI, OFFSET StringAddr
  call WriteString
  pop SI
ENDM
WRITELN MACRO String
  LOCAL AfterString, StringAddr
  jmp AfterString
StringAddr db String, 13, 10, 0
AfterString:
  push SI
  mov SI, OFFSET StringAddr
  call WriteString
  pop SI
ENDM

  org 0
EntryPoint:
  db 0EAh  ;jmp far SEG:OFS    ;Currently we are at 0:7C00
  dw OFFSET AfterData, 7C0h    ;This makes us be at 7C0:AfterData

;Put any data here!
HexTable        db '0123456789ABCDEF'
DiskHeads           dd ?
DiskTracksPerHead   dd ?
DiskSectorsPerTrack dd ?

AfterData: 
  push CS
  pop DS     ; update DS to be 7C0 instead of 0

  mov AX, [EndOfCode]
  call WriteAX
  WRITE " byte SOS Startup"

  mov DL, 0
  call GetDriveStats

  mov AX, SectorBuffer
  mov ES, AX
  mov BX, 0   ;Data buffer will be immediately after us in memory
  mov ECX, 1
  call ReadSector  ;Read the FAT part 1

  mov EAX, ES:[BX]  ;Read number of Allocation Entries
  shl EAX, 2        ;Each is 4 bytes long
  add EAX, 511
  shr EAX, 9        ;Root directory sector = (# of Entries*4+511) / 512
  inc EAX           ;Skip over boot sector

  mov ECX, EAX
  call ReadSector             ;Get the root directory
  mov EAX, ES:[BX+64]         ;Get the cluster of the boot code
  mov ECX, EAX

  mov EDX, ES:[BX+64+4]       ;Get the length of the boot code

  mov AX, BootCodeSector
  mov ES, AX
  mov BX, 0
LoadBootCodeLoop:
  cmp EDX, 0
  jle DoneLoadingBootCode

  WRITE "."

  call ReadSector   ;Get the first sector of the boot code

  inc ECX           ;Next sector
  add BX, 512       ;Next block of memory
  sub EDX, 512
  jmp LoadBootCodeLoop
DoneLoadingBootCode:

  CRLF

  db 0EAh  ;jmp far SEG:OFS    ;Jmp to the start of the BootCode
  dw 0, BootCodeSector         ;at 50h:0

TranslateLogicalSector:          ;In: ECX = Sector  Out: CH, CL, DH, DL
  push EAX EBX
  inc ECX                        ;Sectors start with 1, not 0
  mov EAX, ECX
  xor EDX, EDX
  div [DiskSectorsPerTrack]
  mov CL, DL                     ;Set the sector
  and CL, 63                     ;Top two bits are bits 8&9 of the cylinder
  xor EDX, EDX
  div [DiskTracksPerHead]
  mov CH, DL                     ;Set the track bits 0-7
  mov AL, DH
  shl AL, 6
  or CL, AL                      ;Set bits 8&9 of track
  xor EDX, EDX
  div [DiskHeads]
  mov DH, DL                     ;Set the head
  pop EBX EAX
  ret

ReadSector:   ;In: ES:BX = Data buffer; ECX = Sector
  push AX CX DX
  call TranslateLogicalSector  ;In: ECX  Out: CX, DX
  mov AX, 201h  ;Service 2, Read 1 sector
  mov DL, 0     ;A: drive
  int 13h
  pop DX CX AX
  ret

GetDriveStats:  ;In DL: Drive to get stats for
  mov AH, 8
  int 13h
  xor EAX, EAX
  mov [DiskHeads], EAX
  mov [DiskTracksPerHead], EAX
  mov [DiskSectorsPerTrack], EAX

  mov AL, DH               ;Set the Heads
  mov WORD PTR [DiskHeads], AX

  mov AL, CH               ;Tracks per Head
  mov BYTE PTR [DiskTracksPerHead], AL
  mov AL, CL
  shr AL, 6
  mov BYTE PTR [DiskTracksPerHead+1], AL

  mov AL, CL               ;Sectors Per Track
  and AL, 63
  mov [DiskSectorsPerTrack], EAX

  inc [DiskHeads]
  inc [DiskTracksPerHead]
  mov AX, 50h
  mov ES, AX      ;Sector Buffer
  ret

WriteString:
  push AX BX
WriteStringLoop:
  mov AL, DS:[SI]
  inc SI
  or AL, AL
  jz DoneWriteString

  mov BX, 0
  mov AH, 0Eh
  int 10h
  jmp WriteStringLoop
DoneWriteString:
  pop BX AX
  ret

WriteAX:
  push BX
  mov BL, AH
  shr BL, 4
  call WriteBLNibble
  mov BL, AH
  call WriteBLNibble
  mov BL, AL
  shr BL, 4
  call WriteBLNibble
  mov BL, AL
  call WriteBLNibble
  pop BX
  ret

WriteBLNibble:
  push AX
  and BX, 0Fh
  mov AL, DS:[BX+OFFSET HexTable]
  mov AH, 0Eh
  mov BX, 0
  int 10h
  pop AX
  ret

EndOfCode dw $

ORG 510    ; Make the file 512 bytes long
  DW 0AA55h  ; Add the boot signature
_Text ENDS
  END
