PROGRAM MakeFileSystem;
VAR
  DiskSectorsPerTrack,
  DiskTracksPerHead,
  DiskHeads : WORD;

FUNCTION WriteSector(Sector : WORD; Buffer : POINTER) : BYTE; ASSEMBLER;
ASM
  mov CX, Sector
  mov AX, CX
  xor DX, DX
  div [DiskSectorsPerTrack]
  mov CL, DL                    { Set the sector                            }
  and CL, 63                    { Top two bits are bits 8&9 of the cylinder }
  xor DX, DX
  div [DiskTracksPerHead]
  mov CH, DL                    { Set the track bits 0-7                    }
  mov AL, DH
  ror AL, 1
  ror AL, 1
  and AL, 11000000b
  or CL, AL                     { Set bits 8&9 of track                     }
  xor DX, DX
  div [DiskHeads]
  mov DH, DL                    { Set the head                              }
  inc CL

  mov SI, 4
@TryAgain:
  mov AL, 1
  les BX, Buffer
  mov DL, 0
  mov AH, 3
  int 13h
  jnc @Done
  dec SI
  jnz @TryAgain

@Done:
  mov AL, AH
END;


TYPE
  DWORDArray = ARRAY[0..512 DIV 4] OF LONGINT;
VAR
  Buffer : ^DWORDArray;
  F      : File;
  Error  : BYTE;
  I      : INTEGER;
  BootCodeSize : INTEGER;
  S : STRING;
BEGIN
  DiskHeads := 2;
  DiskTracksPerHead := 80;
  DiskSectorsPerTrack := 18;

  IF ParamCount < 1 THEN
  BEGIN
    WRITELN('Syntax: UpdateBoot <NumSectors> ');
    HALT
  END;

  GETMEM(Buffer, 512);
  ASM
    mov AH, 0
    mov DL, 0
    int 13h     { Reset drive A }
  END;

  VAL(ParamStr(1), BootCodeSize, I);

  FillChar(Buffer^, 512, 0);
  Buffer^[0] := 2880;    { # of entries in FAT }
  FOR I := 1 TO 22 DO
    Buffer^[I] := I+1;   { File allocation table linked together }
  Buffer^[23] := -1;     { End of FAT }
  Buffer^[24] := -1;     { Root directory 1 unit long }
  FOR I := 2 TO BootCodeSize DO
    Buffer^[I+23] := I+23+1;
  Buffer^[BootCodeSize+24] := -1;  { End of boot code chain }

  { Rest are unused }
  IF WriteSector(1, Buffer) <> 0 THEN
    WRITELN('ERROR Writing initial FAT to disk!!!');

  Buffer^[0] := 2;       { 1 entry in root directory }
                         { First file entry...           }
  Buffer^[16] := 25;     { Cluster # of the file's start }
  Buffer^[17] := BootCodeSize*512-1;{ File Length                   }
  Buffer^[18] := 0;      { Flags of this file            }

  S := 'Boot Operations System File'; { Name of this file}
  Move(S[1], Buffer^[19], LENGTH(S)); { Copy into buffer }

  Buffer^[32] := 14;
  Buffer^[33] := $BEEF;
  Buffer^[34] := 0;

  S := 'Extra Test File!!!';
  Move(S[1], Buffer^[35], LENGTH(S)); { Copy into buffer }

  IF WriteSector(24, Buffer) <> 0 THEN
    WRITELN('ERROR Writing root directory to disk!!!');

  WRITELN(BootCodeSize, ' sector allocated to the boot code');
END.